"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AnySchema = void 0;
exports.ArraySchema = ArraySchema;
exports.BooleanSchema = exports.BigIntSchema = void 0;
exports.EqSchema = EqSchema;
Object.defineProperty(exports, "Functor", {
  enumerable: true,
  get: function () {
    return _functor.Functor;
  }
});
exports.InlineSchema = InlineSchema;
exports.IntegerSchema = void 0;
exports.IntersectSchema = IntersectSchema;
exports.NumberSchema = exports.NullSchema = exports.NeverSchema = void 0;
exports.ObjectSchema = ObjectSchema;
exports.OptionalSchema = OptionalSchema;
Object.defineProperty(exports, "RAW", {
  enumerable: true,
  get: function () {
    return Spec.RAW;
  }
});
exports.RecordSchema = RecordSchema;
exports.SymbolSchema = exports.StringSchema = void 0;
exports.TupleSchema = TupleSchema;
exports.UndefinedSchema = void 0;
exports.UnionSchema = UnionSchema;
exports.VoidSchema = exports.UnknownSchema = void 0;
Object.defineProperty(exports, "fold", {
  enumerable: true,
  get: function () {
    return _functor.fold;
  }
});
Object.defineProperty(exports, "is", {
  enumerable: true,
  get: function () {
    return Spec.is;
  }
});
Object.defineProperty(exports, "isRequired", {
  enumerable: true,
  get: function () {
    return _properties.isRequired;
  }
});
Object.defineProperty(exports, "minItems", {
  enumerable: true,
  get: function () {
    return _items.minItems;
  }
});
Object.defineProperty(exports, "property", {
  enumerable: true,
  get: function () {
    return _properties.property;
  }
});
Object.defineProperty(exports, "unfold", {
  enumerable: true,
  get: function () {
    return _functor.unfold;
  }
});
Object.defineProperty(exports, "wrapOptional", {
  enumerable: true,
  get: function () {
    return _properties.wrapOptional;
  }
});
var _registry = /*#__PURE__*/require("@traversable/registry");
var _items = /*#__PURE__*/require("./items.js");
var _properties = /*#__PURE__*/require("./properties.js");
var Spec = /*#__PURE__*/_interopRequireWildcard(/*#__PURE__*/require("./specification.js"));
var _functor = /*#__PURE__*/require("./functor.js");
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/** @internal */
const Object_keys = globalThis.Object.keys;
function applyTupleOptionality(xs, {
  min,
  max
}) {
  return min === max ? xs.map(_properties.getSchema) : [...xs.slice(0, min).map(_properties.getSchema), ...xs.slice(min).map(_properties.getSchema)];
}
const NeverSchema = exports.NeverSchema = {
  toJsonSchema() {
    return void 0;
  }
};
const VoidSchema = exports.VoidSchema = {
  toJsonSchema() {
    return void 0;
  }
};
const UndefinedSchema = exports.UndefinedSchema = {
  toJsonSchema() {
    return void 0;
  }
};
const SymbolSchema = exports.SymbolSchema = {
  toJsonSchema() {
    return void 0;
  }
};
const BigIntSchema = exports.BigIntSchema = {
  toJsonSchema() {
    return void 0;
  }
};
function InlineSchema(_) {
  return {
    toJsonSchema: () => void 0
  };
}
/* * * * * * * * *
 *                *
 *   data types   *
 *                *
  * * * * * * * * */
const AnySchema = exports.AnySchema = {
  toJsonSchema() {
    return Spec.RAW.any;
  }
};
const UnknownSchema = exports.UnknownSchema = {
  toJsonSchema() {
    return Spec.RAW.any;
  }
};
const NullSchema = exports.NullSchema = {
  toJsonSchema() {
    return Spec.RAW.null;
  }
};
const BooleanSchema = exports.BooleanSchema = {
  toJsonSchema() {
    return Spec.RAW.boolean;
  }
};
const IntegerSchema = exports.IntegerSchema = {
  toJsonSchema() {
    return Spec.RAW.integer;
  }
};
const NumberSchema = exports.NumberSchema = {
  toJsonSchema() {
    return Spec.RAW.number;
  }
};
const StringSchema = exports.StringSchema = {
  toJsonSchema() {
    return Spec.RAW.string;
  }
};
function EqSchema(value) {
  return {
    toJsonSchema() {
      return {
        const: value
      };
    }
  };
}
function OptionalSchema(x) {
  function toJsonSchema() {
    return (0, _properties.getSchema)(x);
  }
  toJsonSchema[_registry.symbol.optional] = (0, _properties.wrapOptional)(x);
  return {
    toJsonSchema
  };
}
function ArraySchema(x) {
  return {
    toJsonSchema() {
      return {
        type: 'array',
        items: (0, _properties.getSchema)(x)
      };
    }
  };
}
function RecordSchema(x) {
  return {
    toJsonSchema() {
      return {
        type: 'object',
        additionalProperties: (0, _properties.getSchema)(x)
      };
    }
  };
}
function UnionSchema(xs) {
  return {
    toJsonSchema() {
      return {
        anyOf: xs.map(_properties.getSchema)
      };
    }
  };
}
function IntersectSchema(xs) {
  return {
    toJsonSchema() {
      return {
        allOf: xs.map(_properties.getSchema)
      };
    }
  };
}
function TupleSchema(xs) {
  const min = (0, _items.minItems)(xs);
  const max = xs.length;
  return {
    toJsonSchema() {
      return {
        type: 'array',
        additionalItems: false,
        items: applyTupleOptionality(xs, {
          min,
          max
        }),
        minItems: min,
        maxItems: max
      };
    }
  };
}
function ObjectSchema(xs) {
  const required = Object_keys(xs).filter((0, _properties.isRequired)(xs));
  return {
    toJsonSchema() {
      return {
        type: 'object',
        required,
        properties: _registry.fn.map(xs, (0, _properties.property)(required))
      };
    }
  };
}
//# sourceMappingURL=jsonSchema.js.map