import type { Returns } from '@traversable/registry';
import { symbol } from '@traversable/registry';
import type { MinItems } from './items.js';
import type { RequiredKeys } from './properties.js';
import * as Spec from './specification.js';
export type { Unary, Free, Nullary, } from './specification.js';
export { is, RAW, } from './specification.js';
export { Functor, fold, unfold, } from './functor.js';
export type { RequiredKeys } from './properties.js';
export { isRequired, property, wrapOptional, } from './properties.js';
export type { MinItems } from './items.js';
export { minItems } from './items.js';
export type { JsonSchema, LowerBound, };
export { SymbolSchema, VoidSchema, UndefinedSchema, BigIntSchema, NeverSchema, InlineSchema, UnknownSchema, AnySchema, NullSchema, BooleanSchema, IntegerSchema, NumberSchema, StringSchema, EqSchema, OptionalSchema, ArraySchema, RecordSchema, TupleSchema, UnionSchema, IntersectSchema, ObjectSchema, };
type JsonSchema<T = never> = [T] extends [never] ? Spec.JsonSchema : Spec.Unary<T>;
interface LowerBound {
    toJsonSchema(): JsonSchema | undefined;
}
interface NeverSchema {
    toJsonSchema(): undefined;
}
declare const NeverSchema: NeverSchema;
interface VoidSchema {
    toJsonSchema(): undefined;
}
declare const VoidSchema: VoidSchema;
interface UndefinedSchema {
    toJsonSchema(): undefined;
}
declare const UndefinedSchema: UndefinedSchema;
interface SymbolSchema {
    toJsonSchema(): undefined;
}
declare const SymbolSchema: SymbolSchema;
interface BigIntSchema {
    toJsonSchema(): undefined;
}
declare const BigIntSchema: BigIntSchema;
interface InlineSchema<S> {
    toJsonSchema: () => void;
}
declare function InlineSchema<S>(_: S): InlineSchema<S>;
declare const AnySchema: AnySchema;
interface AnySchema {
    toJsonSchema(): typeof Spec.RAW.any;
}
declare const UnknownSchema: UnknownSchema;
interface UnknownSchema {
    toJsonSchema(): typeof Spec.RAW.any;
}
declare const NullSchema: NullSchema;
interface NullSchema {
    toJsonSchema(): typeof Spec.RAW.null;
}
declare const BooleanSchema: BooleanSchema;
interface BooleanSchema {
    toJsonSchema(): typeof Spec.RAW.boolean;
}
declare const IntegerSchema: IntegerSchema;
interface IntegerSchema {
    toJsonSchema(): typeof Spec.RAW.integer;
}
declare const NumberSchema: NumberSchema;
interface NumberSchema {
    toJsonSchema(): typeof Spec.RAW.number;
}
declare const StringSchema: StringSchema;
interface StringSchema {
    toJsonSchema(): typeof Spec.RAW.string;
}
interface EqSchema<S> {
    toJsonSchema(): {
        const: S;
    };
}
declare function EqSchema<V>(value: V): EqSchema<V>;
interface OptionalSchema<S> {
    toJsonSchema: {
        [symbol.optional]: number;
        (): Returns<S['toJsonSchema' & keyof S]>;
    };
}
declare function OptionalSchema<S>(x: S): OptionalSchema<S>;
interface ArraySchema<S> {
    toJsonSchema(): {
        type: 'array';
        items: Returns<S['toJsonSchema' & keyof S]>;
    };
}
declare function ArraySchema<S>(schema: S): ArraySchema<S>;
interface RecordSchema<S> {
    toJsonSchema(): {
        type: 'object';
        additionalProperties: Returns<S['toJsonSchema' & keyof S]>;
    };
}
declare function RecordSchema<S>(schema: S): RecordSchema<S>;
interface UnionSchema<S> {
    toJsonSchema(): {
        anyOf: {
            [I in keyof S]: Returns<S[I]['toJsonSchema' & keyof S[I]]>;
        };
    };
}
declare function UnionSchema<S extends readonly unknown[]>(schemas: S): UnionSchema<S>;
interface IntersectSchema<S> {
    toJsonSchema(): {
        allOf: {
            [I in keyof S]: Returns<S[I]['toJsonSchema' & keyof S[I]]>;
        };
    };
}
declare function IntersectSchema<S extends readonly unknown[]>(schemas: S): IntersectSchema<S>;
interface TupleSchema<S> {
    toJsonSchema(): {
        type: 'array';
        items: {
            [I in keyof S]: Returns<S[I]['toJsonSchema' & keyof S[I]]>;
        };
        additionalItems: false;
        minItems: MinItems<S>;
        maxItems: S['length' & keyof S];
    };
}
declare function TupleSchema<S extends readonly unknown[]>(schemas: readonly [...S]): TupleSchema<S>;
interface ObjectSchema<S, KS extends RequiredKeys<S> = RequiredKeys<S>> {
    toJsonSchema(): {
        type: 'object';
        required: {
            [I in keyof KS]: KS[I] & string;
        };
        properties: {
            [K in keyof S]: Returns<S[K]['toJsonSchema' & keyof S[K]]>;
        };
    };
}
declare function ObjectSchema<S extends {
    [x: string]: unknown;
}>(schemas: S): ObjectSchema<S>;
//# sourceMappingURL=jsonSchema.d.ts.map